@@ -12,6 +12,7 @@
 require "highline/system_extensions"
 require "highline/question"
 require "highline/menu"
+require "highline/color_scheme"
 require "erb"
 require "optparse"
 require "stringio"
@@ -28,7 +29,7 @@ require "abbrev"
 #
 class HighLine
   # The version of the installed library.
-  VERSION = "1.2.2".freeze
+  VERSION = "1.4.0".freeze
   
   # An internal HighLine error.  User code does not need to trap this.
   class QuestionError < StandardError
@@ -47,6 +48,37 @@ class HighLine
   def self.use_color?
     @@use_color
   end
+  
+  # The setting used to disable EOF tracking.
+  @@track_eof = true
+  
+  # Pass +false+ to _setting_ to turn off HighLine's EOF tracking.
+  def self.track_eof=( setting )
+    @@track_eof = setting
+  end
+  
+  # Returns true if HighLine is currently tracking EOF for input.
+  def self.track_eof?
+    @@track_eof
+  end
+
+  # The setting used to control color schemes.
+  @@color_scheme = nil
+
+  # Pass ColorScheme to _setting_ to turn set a HighLine color scheme.
+  def self.color_scheme=( setting )
+    @@color_scheme = setting
+  end
+
+  # Returns the current color scheme.
+  def self.color_scheme
+    @@color_scheme
+  end
+
+  # Returns +true+ if HighLine is currently using a color scheme.
+  def self.using_color_scheme?
+    not @@color_scheme.nil?
+  end
 
   #
   # Embed in a String to clear all previous ANSI sequences.  This *MUST* be 
@@ -55,6 +87,10 @@ class HighLine
   CLEAR      = "\e[0m"
   # An alias for CLEAR.
   RESET      = CLEAR
+  # Erase the current line of terminal output.
+  ERASE_LINE = "\e[K"
+  # Erase the character under the cursor.
+  ERASE_CHAR = "\e[P"
   # The start of an ANSI bold sequence.
   BOLD       = "\e[1m"
   # The start of an ANSI dark sequence.  (Terminal support uncommon.)
@@ -168,9 +204,11 @@ class HighLine
     @question ||= Question.new(question, answer_type, &details)
     
     return gather if @question.gather
-    
-    # readline() needs to handle it's own output
-    say(@question) unless @question.readline
+  
+    # readline() needs to handle it's own output, but readline only supports 
+    # full line reading.  Therefore if @question.echo is anything but true, 
+    # the prompt will not be issued. And we have to account for that now.
+    say(@question) unless (@question.readline and @question.echo == true)
     begin
       @answer = @question.answer_or_default(get_response)
       unless @question.valid_answer?(@answer)
@@ -243,7 +281,7 @@ class HighLine
     # Set _answer_type_ so we can double as the Question for ask().
     @menu.answer_type = if @menu.shell
       lambda do |command|    # shell-style selection
-        first_word = command.split.first
+        first_word = command.to_s.split.first || ""
 
         options = @menu.options
         options.extend(OptionParser::Completion)
@@ -287,13 +325,15 @@ class HighLine
     return string unless self.class.use_color?
     
     colors.map! do |c|
-      if c.is_a?(Symbol)
+      if self.class.using_color_scheme? and self.class.color_scheme.include? c
+        self.class.color_scheme[c]
+      elsif c.is_a? Symbol
         self.class.const_get(c.to_s.upcase)
       else
         c
       end
     end
-    "#{colors.join}#{string}#{CLEAR}"
+    "#{colors.flatten.join}#{string}#{CLEAR}"
   end
   
   # 
@@ -550,7 +590,8 @@ class HighLine
 
       answer
     else
-      raise EOFError, "The input stream is exhausted." if @input.eof?
+      raise EOFError, "The input stream is exhausted." if @@track_eof and
+                                                          @input.eof?
 
       @question.change_case(@question.remove_whitespace(@input.gets))
     end
@@ -575,19 +616,46 @@ class HighLine
         raw_no_echo_mode if stty = CHARACTER_MODE == "stty"
         
         line = ""
+        backspace_limit = 0
         begin
+
           while character = (stty ? @input.getc : get_character(@input))
-            line << character.chr
+            # honor backspace and delete
+            if character == 127 or character == 8
+              line.slice!(-1, 1)
+              backspace_limit -= 1
+            else
+              line << character.chr
+              backspace_limit = line.size
+            end
             # looking for carriage return (decimal 13) or
             # newline (decimal 10) in raw input
             break if character == 13 or character == 10 or
                      (@question.limit and line.size == @question.limit)
-            @output.print(@question.echo) if @question.echo != false
+            if @question.echo != false
+              if character == 127 or character == 8 
+                  # only backspace if we have characters on the line to
+                  # eliminate, otherwise we'll tromp over the prompt
+                  if backspace_limit >= 0 then
+                    @output.print("\b#{ERASE_CHAR}")
+                  else 
+                      # do nothing
+                  end
+              else
+                @output.print(@question.echo)
+              end
+              @output.flush
+            end
           end
-          say("\n")
         ensure
           restore_mode if stty
         end
+        if @question.overwrite
+          @output.print("\r#{ERASE_LINE}")
+          @output.flush
+        else
+          say("\n")
+        end
         
         @question.change_case(@question.remove_whitespace(line))
       end
@@ -595,14 +663,19 @@ class HighLine
       @question.change_case(@input.getc.chr)
     else
       response = get_character(@input).chr
-      echo = if @question.echo == true
-        response
-      elsif @question.echo != false
-        @question.echo
+      if @question.overwrite
+        @output.print("\r#{ERASE_LINE}")
+        @output.flush
       else
-        ""
+        echo = if @question.echo == true
+          response
+        elsif @question.echo != false
+          @question.echo
+        else
+          ""
+        end
+        say("#{echo}\n")
       end
-      say("#{echo}\n")
       @question.change_case(response)
     end
   end
