@@ -132,6 +132,27 @@ class TestHighLine < Test::Unit::TestCase
     assert_equal(2, answer)
     assert_equal("Select an option (1, 2 or 3):  *\n", @output.string)
   end
+
+  def test_backspace_does_not_enter_prompt
+      @input << "\b\b"
+      @input.rewind
+      answer = @terminal.ask("Please enter your password: ") do |q| 
+          q.echo = "*" 
+      end
+      assert_equal("", answer)
+      assert_equal("Please enter your password: \n",@output.string)
+  end
+  
+  def test_readline_on_non_echo_question_has_prompt
+    @input << "you can't see me"
+    @input.rewind
+    answer = @terminal.ask("Please enter some hidden text: ") do |q|
+      q.readline = true
+      q.echo = "*"
+    end
+    assert_equal("you can't see me", answer)
+    assert_equal("Please enter some hidden text: ****************\n",@output.string)
+  end
   
   def test_character_reading
     # WARNING:  This method does NOT cover Unix and Windows savvy testing!
@@ -143,7 +164,7 @@ class TestHighLine < Test::Unit::TestCase
     end
     assert_equal(1, answer)
   end
-  
+
   def test_color
     @terminal.say("This should be <%= BLUE %>blue<%= CLEAR %>!")
     assert_equal("This should be \e[34mblue\e[0m!\n", @output.string)
@@ -771,6 +792,20 @@ class TestHighLine < Test::Unit::TestCase
     assert_equal(("-=" * 40 + "\n") + ("-=" * 10 + "\n"), @output.string)
   end
   
+  def test_track_eof
+    assert_raise(EOFError) { @terminal.ask("Any input left?  ") }
+    
+    # turn EOF tracking
+    old_setting = HighLine.track_eof?
+    assert_nothing_raised(Exception) { HighLine.track_eof = false }
+    begin
+      @terminal.ask("And now?  ")  # this will still blow up, nothing available
+    rescue
+      assert_not_equal(EOFError, $!.class)  # but HighLine's safe guards are off
+    end
+    HighLine.track_eof = old_setting
+  end
+  
   def test_version
     assert_not_nil(HighLine::VERSION)
     assert_instance_of(String, HighLine::VERSION)
